From c6db366cfd8e876299eadc2eae75c696facf52ca Mon Sep 17 00:00:00 2001
From: Michael Paquier <michael@otacoo.com>
Date: Sat, 6 Dec 2014 22:06:59 +0900
Subject: [PATCH 3/3] use hack to calculate user and system time used for
 processes

---
 src/backend/tcop/postgres.c | 22 +++++++++++++++++++++-
 1 file changed, 21 insertions(+), 1 deletion(-)

diff --git a/src/backend/tcop/postgres.c b/src/backend/tcop/postgres.c
index cc62b2c..7f3f0c7 100644
--- a/src/backend/tcop/postgres.c
+++ b/src/backend/tcop/postgres.c
@@ -33,6 +33,8 @@
 #include <sys/resource.h>
 #endif
 
+#include <sys/resource.h>
+
 #ifndef HAVE_GETRUSAGE
 #include "rusagestub.h"
 #endif
@@ -175,6 +177,10 @@ static bool RecoveryConflictPending = false;
 static bool RecoveryConflictRetryable = true;
 static ProcSignalReason RecoveryConflictReason;
 
+/* Amount of user and system time used, tracked at start */
+static struct timeval user_time;
+static struct timeval system_time;
+
 /* ----------------------------------------------------------------
  *		decls for routines only used in this file
  * ----------------------------------------------------------------
@@ -3535,6 +3541,12 @@ PostgresMain(int argc, char *argv[],
 	StringInfoData input_message;
 	sigjmp_buf	local_sigjmp_buf;
 	volatile bool send_ready_for_query = true;
+	struct rusage r;
+
+	/* Get start usage for reference point */
+	getrusage(RUSAGE_SELF, &r);
+	memcpy((char *) &user_time, (char *) &r.ru_utime, sizeof(user_time));
+	memcpy((char *) &system_time, (char *) &r.ru_stime, sizeof(system_time));
 
 	/*
 	 * Initialize globals (already done if under postmaster, but not if
@@ -4217,6 +4229,14 @@ PostgresMain(int argc, char *argv[],
 			case 'X':
 			case EOF:
 
+				/* Get stop status of process and log comparison with start */
+				getrusage(RUSAGE_SELF, &r);
+				elog(LOG,"user diff: %ld.%06ld, system diff: %ld.%06ld",
+					 (long) (r.ru_utime.tv_sec - user_time.tv_sec),
+					 (long) (r.ru_utime.tv_usec - user_time.tv_usec),
+					 (long) (r.ru_stime.tv_sec - system_time.tv_sec),
+					 (long) (r.ru_stime.tv_usec - system_time.tv_usec));
+
 				/*
 				 * Reset whereToSendOutput to prevent ereport from attempting
 				 * to send any more messages to client.
@@ -4250,7 +4270,7 @@ PostgresMain(int argc, char *argv[],
 						 errmsg("invalid frontend message type %d",
 								firstchar)));
 		}
-	}							/* end of input-reading loop */
+	} /* end of input-reading loop */
 }
 
 /*
-- 
2.2.0

